"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const passport_strategy_runner_1 = require("passport-strategy-runner");
function isPassportAuthenticator(obj) {
    return obj.use && obj.authenticate && obj.initialize && obj.session;
}
function isString(obj) {
    return typeof (obj) === 'string';
}
/* istanbul ignore next */
function assertNever(x) {
    throw new Error("Unexpected object: " + JSON.stringify(x));
}
function defaultConverter(user) {
    return {
        user: user,
        roles: user.roles,
        scopes: user.scopes
    };
}
function defaultIsPresent(context, info) {
    let answer = true; // Assume worst case.
    if (info.name && info.in === 'header' && !(info.name.toLowerCase() in context.req.headers)) {
        answer = false;
    }
    else if (info.name && info.in === 'query' && !context.req.url.includes(info.name)) {
        answer = false;
    }
    else if (info.scheme) {
        let authorization = context.req.headers['authorization'];
        if (authorization === null || authorization === undefined) {
            answer = false;
        }
        else {
            if (!Array.isArray(authorization)) {
                authorization = [authorization];
            }
            answer = authorization.some(authHeader => authHeader.slice(0, info.scheme.length).toLowerCase() === info.scheme.toLowerCase());
        }
    }
    return answer;
}
/**
 * Temporarily clears the user.  We call this before every passport
 * authenticator, because passport's session strategy will just set
 * req.user and then call `pass()` instead of `success()`.
 *
 * @param req - The incoming HTTP request.
 * @param originalCallback - The original callback.
 * @returns - A new callback which will restore `req.user` and then call the
 *   original callback.
 */
function clearUser(req, originalCallback) {
    const origUser = req.user;
    req.user = undefined;
    return (err, result) => {
        req.user = origUser;
        originalCallback(err, result);
    };
}
function generateSuccessResult(pluginContext, converter, user) {
    const result = Object.assign({ type: 'success' }, converter(user, pluginContext));
    return result;
}
function getTypeOnFailure(options, pluginContext, info) {
    let type;
    if (!options.isPresent) {
        type = defaultIsPresent(pluginContext, info) ? 'invalid' : 'missing';
    }
    else if (!options.isPresent(pluginContext, info)) {
        type = 'missing';
    }
    else {
        type = 'invalid';
    }
    return type;
}
/**
 * Create a new Exegesis authenticator from a passport instance.
 *
 * @param passport - The passport instance to call into.  This should be installed
 *   in your middleware chain before exegesis with `app.use(passport.initialize())`.
 * @param strategyName - The name of the passport strategy to use.  This strategy
 *   should be registered with passport with `passport.use(...)`.
 * @param options - options to pass to strategy.
 * @param converter - A function to convert Passport users into
 *   `{user, roles, scopes}` objects.  If not defined, `user.roles` will be used
 *   as roles, and `user.scopes` will be used as scopes.
 * @returns - An Exegesis authenticator.
 */
function makePassportAuthenticator(passport, strategyName, options = {}) {
    const converter = options.convert || defaultConverter;
    return function passportAuthenticator(pluginContext, info, done) {
        const req = pluginContext.req;
        const origDone = clearUser(req, done);
        done = (err, result) => {
            if (err) {
                origDone(err);
            }
            else if (!(result && result.type === 'success') && req.user) {
                // Passport didn't give us a user, but it did set req.user.
                // The session middleware does this.
                origDone(null, generateSuccessResult(pluginContext, converter, req.user));
            }
            else {
                origDone(err, result);
            }
        };
        passport.authenticate(strategyName, options.passportOptions || {}, (err, user, challenge, status) => {
            if (err) {
                done(err);
            }
            else if (user) {
                done(null, generateSuccessResult(pluginContext, converter, user));
            }
            else {
                const type = getTypeOnFailure(options, pluginContext, info);
                const result = { type, status };
                if (challenge && isString(challenge)) {
                    result.challenge = challenge;
                }
                else if (challenge && challenge.message) {
                    result.message = challenge.message;
                }
                done(null, result);
            }
        })(pluginContext.req, pluginContext.origRes, done);
    };
}
/**
 * Create a new Exegesis authenticator from a passport strategy.
 *
 * @param strategy - An instance of a Passport strategy to call.  Passport
 *   does not need to be registered as a middleware for this to work.
 * @param options - options to pass to strategy.
 * @param converter - A function to convert Passport users into
 *   `{user, roles, scopes}` objects.  If not defined, `user.roles` will be used
 *   as roles, and `user.scopes` will be used as scopes.
 * @returns - An Exegesis authenticator.
 */
function makeStrategyRunner(strategy, options = {}) {
    return function passportStrategyAuthenticator(pluginContext, info, done) {
        const req = pluginContext.req;
        const converter = options.convert || defaultConverter;
        done = clearUser(req, done);
        passport_strategy_runner_1.runStrategy(strategy, pluginContext.req, options.passportOptions, (err, result) => {
            if (err || !result) {
                return done(err);
            }
            switch (result.type) {
                case 'success': {
                    done(null, generateSuccessResult(pluginContext, converter, result.user));
                    break;
                }
                case 'pass':
                    if (req.user) {
                        // Passport didn't give us a user, but it did set req.user.
                        // The session middleware does this.
                        done(null, generateSuccessResult(pluginContext, converter, req.user));
                    }
                    else {
                        done(null, undefined);
                    }
                    break;
                case 'fail':
                    const answer = {
                        type: getTypeOnFailure(options, pluginContext, info),
                        status: result.status,
                        challenge: result.challenge,
                        message: result.message
                    };
                    done(null, answer);
                    break;
                case 'redirect': {
                    const { res } = pluginContext;
                    res.setStatus(result.status)
                        .set('location', result.url)
                        .setBody('');
                    done(null, undefined);
                    break;
                }
                /* istanbul ignore next */
                default:
                    assertNever(result);
            }
        });
    };
}
function exegesisPassport(a, b, c) {
    if (isPassportAuthenticator(a) && isString(b)) {
        return makePassportAuthenticator(a, b, c);
    }
    else {
        return makeStrategyRunner(a, b);
    }
}
exports.exegesisPassport = exegesisPassport;
exports.default = exegesisPassport;
//# sourceMappingURL=index.js.map